;; SparkFun Joystick
;;
;; Copyright 2013 - By Michael Kohn
;; http://www.mikekohn.net/
;; mike at mikekohn.net
;;
;; Use a SparkFun joystick kit to send Syma S107 data

.include "msp430x2xx.inc"

; 2.0 ms = 152 interrupts
; 0.3 ms = 23 interrupts
; 0.6 ms = 46 interrupts

; HH YAW PTCH THROTTLE YAW_CORRECT
; YAW = 0-126  63
; PITCH = 0-126 63
; THROTTLE = 0-126
; CORR = 0-126 63  (D3-D6 buttons)

RAM equ 0x0200
HEADER equ 152
SHORT equ 23
LONG equ 46

YAW equ RAM
PITCH equ RAM+1
THROTTLE equ RAM+2
CORR equ RAM+3

;  r4 = state (0=idle, 1=header_on, 2=header_off, 3=first half, 4=second)
;  r5 = interupt count
;  r6 = pointer to next byte going out
;  r7 = current byte
;  r8 = bit count
;  r9 = bit time len
; r10 = temp
; r11 =
; r12 = extra sends after idle controller
; r13 = interrupt routine
; r14 =
; r15 =

  .org 0xf800
start:
  ;; Turn off watchdog
  mov.w #(WDTPW|WDTHOLD), &WDTCTL

  ;; Please don't interrupt me
  dint

  ;; r13 points to which interrupt routine should be called
  mov.w #led_off, r13

  ;; Set up stack pointer
  mov.w #0x0280, SP

  ;; Set MCLK to 16 MHz with DCO 
  mov.b #DCO_4, &DCOCTL
  mov.b #RSEL_15, &BCSCTL1
  mov.b #0, &BCSCTL2

.if 0
  ;; Set MCLK to 16 MHz external crystal
  bic.w #OSCOFF, SR
  bis.b #XTS, &BCSCTL1
  mov.b #LFXT1S_3, &BCSCTL3
  ;mov.b #LFXT1S_3|XCAP_1, &BCSCTL3
test_osc:
  bic.b #OFIFG, &IFG1
  mov.w #0x00ff, r15
dec_again:
  dec r15
  jnz dec_again
  bit.b #(OFIFG), &IFG1
  jnz test_osc
  mov.b #(SELM_3|SELS), &BCSCTL2
.endif

  ;; Set up output pins
  ;; P1.0 = D6
  ;; P1.1 = D5
  ;; P1.2 = D4
  ;; P1.3 = D3
  ;; P1.4 = Reset (Just a button.. doesn't reset anything)
  ;; P1.5 = Joystick push-button
  ;; P1.6 = Pitch
  ;; P1.7 = Yaw
  ;; P2.6 = Data Out on IR LED
  mov.b #0x00, &P1DIR
  mov.b #0x3f, &P1OUT
  mov.b #0x3f, &P1REN
  mov.b #64, &P2DIR
  mov.b #64, &P2OUT
  mov.b #0, &P2SEL

  ;; Set up A/D
  mov.w #ADC10ON|ADC10SHT_3, &ADC10CTL0
  mov.w #INCH_6|ADC10SSEL_0, &ADC10CTL1
  mov.w #0xc0, &ADC10AE0

  ;; Set up Timer
  mov.w #210, &TACCR0
  mov.w #(TASSEL_2|MC_1), &TACTL ; SMCLK, DIV1, COUNT to TACCR0
  mov.w #CCIE, &TACCTL0
  mov.w #0, &TACCTL1

  mov.b #63, &YAW
  mov.b #63, &PITCH
  mov.b #0, &THROTTLE
  mov.b #0, &CORR

  mov.b #0, r12

  ;; Okay, I can be interrupted now
  eint

main:
  mov.b &P1IN, r10 
  xor.b #0x3f, r10
  and.b #0x3f, r10   ; wtf?
  mov.b r10, &CORR


  mov.w #INCH_7|ADC10SSEL_0|ADC10DIV_7, &ADC10CTL1
  bic.w #ADC10IFG, &ADC10CTL0
  bis.w #ENC|ADC10SC, &ADC10CTL0
adc10_busy_pitch:
  bit.w #ADC10BUSY, &ADC10CTL1
  jnz adc10_busy_pitch
  ;bit.w #ADC10IFG, &ADC10CTL0
  ;jz adc10_busy_pitch
  mov.w &ADC10MEM, r10
  rra r10
  rra r10
  rra r10
  mov.b #127, r11
  sub.b r10, r11
  mov.b r11, &PITCH
  bic.w #ENC, &ADC10CTL0

  mov.w #INCH_6|ADC10SSEL_0|ADC10DIV_7, &ADC10CTL1
  bic.w #ADC10IFG, &ADC10CTL0
  bis.w #ENC|ADC10SC, &ADC10CTL0
adc10_busy_yaw:
  bit.w #ADC10BUSY, &ADC10CTL1
  jnz adc10_busy_yaw
  ;bit.w #ADC10IFG, &ADC10CTL0
  ;jz adc10_busy_yaw
  mov.w &ADC10MEM, r10
  rra r10
  rra r10
  rra r10
  mov.b #127, r11
  sub.b r10, r11
  mov.b r11, &YAW
  bic.w #ENC, &ADC10CTL0

  cmp.b #0, &CORR
  jne do_send_command
  cmp.b #67, &YAW
  jhs do_send_command
  cmp.b #67, &PITCH
  jhs do_send_command

  cmp.b #62, &YAW
  jlo do_send_command
  cmp.b #62, &PITCH
  jlo do_send_command

  cmp.b #0, r12
  jeq skip_send_command
  dec.b r12
  call #send_command
  jmp skip_send_command

do_send_command:
  call #send_command
  mov.b #4, r12

skip_send_command:
  mov #0, r5
delay:
  cmp.w #152*10, r5
  jlo delay

  jmp main

send_command:
  mov.w #led_on, r13
  mov.w #0, r5
wait_header_on:
  cmp.w #HEADER, r5
  jl wait_header_on

  mov.w #led_off, r13
  mov.w #0, r5
wait_header_off:
  cmp.w #HEADER, r5
  jl wait_header_off

  mov.w #RAM, r6
send_next_byte:
  mov.b @r6+, r7
  mov.b #8, r8
send_next_bit:
  mov.w #LONG, r9
  rla.b r7
  jc it_was_a_1
  mov.w #SHORT, r9
it_was_a_1:

  mov.w #led_on, r13
  mov.w #0, r5
wait_on_first_half:
  cmp.w #SHORT, r5
  jl wait_on_first_half

  mov.w #led_off, r13
  mov.w #0, r5
wait_on_second_half:
  cmp.w r9, r5
  jl wait_on_second_half

  dec.b r8
  jnz send_next_bit

  cmp.w #RAM+4, r6
  jne send_next_byte

  mov.w #led_on, r13
  mov.w #0, r5
wait_on_stop_bit:
  cmp.w #SHORT, r5
  jl wait_on_stop_bit

  mov.w #led_off, r13
  bic.b #64, &P2OUT
  ret

.if 0
time_delay:
  mov.w #10, r11
time_delay_dec_outer:
  mov.w #0, r10
time_delay_dec:
  dec.w r10
  jnz time_delay_dec
  dec.w r11
  jnz time_delay_dec_outer
  ret
.endif


timer_interrupt:
  br r13

led_on:
  xor.b #64, &P2OUT
  inc.w r5
  reti

led_off:
  bic.b #64, &P2OUT
  inc.w r5
  reti

  org 0xffe8
vectors:
  dw 0
  dw 0
  dw 0
  dw 0
  dw 0
  dw timer_interrupt       ; Timer_A2 TACCR0, CCIFG
  dw 0
  dw 0
  dw 0
  dw 0
  dw 0
  dw start                 ; Reset



